local sound = require "sound"
local game = {}
settings = require "settings"
function game.init()
	settings.init()
	_WORLD_SIZE_X = 650
	_WORLD_SIZE_Y = 650
	_GROUND_SIZE_X = _WORLD_SIZE_X
	_GROUND_SIZE_Y = 25
	_WALL_SIZE_X = 25
	_WALL_SIZE_Y = _WORLD_SIZE_Y
	_STAR_OFFSET_X = -290
	_STAR_OFFSET_Y = 700
	def_font = love.graphics.newFont(14)
	font = love.graphics.newImageFont("graphics/testfont.png", " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.,!?-+/():;%&`'*#=[]\"")
	endmessage = ""
	love.window.setTitle("Yet Another Invasion")
	love.physics.setMeter(64)
	-- color definitions
	_RED = {255, 0, 0}
	_GREEN = {0, 255, 0}
	_YELLOW = {255, 255, 0}
	_PINK = {255, 192, 203}
	_MAGENTA = {255, 0, 255}
	_BLUE = {65, 105, 225}
	_WHITE = {255, 255, 255}
	_BLACK = {0, 0, 0}
	_GRAY = {160, 160, 160}
	_SKYCOLOR = {10, 10, 15}
	colors = {}
	table.insert(colors, _RED)
	table.insert(colors, _GREEN)
	table.insert(colors, _YELLOW)
	table.insert(colors, _PINK)
	table.insert(colors, _MAGENTA)
	table.insert(colors, _BLUE)
	-- graphics ressources
	images = {}
	images.player = love.graphics.newImage("graphics/player.png")
	images.explosion = love.graphics.newImage("graphics/exp.png")
	images.blast = love.graphics.newImage("graphics/blast.png")
	images.bullet = {}
	images.bullet.happy = love.graphics.newImage("graphics/bullet_happy.png")
	images.bullet.ow = love.graphics.newImage("graphics/bullet_ow.png")
	images.ammo = love.graphics.newImage("graphics/ammo.png")
	images.stars = love.graphics.newImage("graphics/stars.png")
	images.aliens = {}
	table.insert(images.aliens, love.graphics.newImage("graphics/alien1.png"))
	table.insert(images.aliens, love.graphics.newImage("graphics/alien2.png"))
	images.alien_mother = love.graphics.newImage("graphics/alien_mother.png")
	images.alien_mother_bullet = love.graphics.newImage("graphics/bullet_alien.png")
	images.invisible = love.graphics.newImage("graphics/invisible.png")
	anims = {}
	anims.explosion = {}
	-- sound ressources
	sound.init()
	-- types
	types = {}
	types.world = 0
	types.player = 1
	types.bullet = 2
	types.alien = 3
	types.mothership = 4
	types.alienBullet = 5
	game.restartGame()
end
function game.update(dt)
	if(not pause) then
		world:update(dt)
		for i, bullet in ipairs(bullets) do
			bullet.age = bullet.age + dt
			if bullet.lastHitTime < love.timer.getTime() - .5 then
				bullet.image = images.bullet.happy
			end
			if bullet.blast.sizeMod > 0 then
				bullet.blast.sizeMod = bullet.blast.sizeMod - .1
			end
			bullet.blast.age = bullet.blast.age + dt
			if bullet.blast.age > .2 then
				bullet.blast.image = images.invisible
			end
			if bullet.age > 2 then
				bullet.body:setBullet(false)
			end
		end
		-- aliens
		for i, alien in ipairs(aliens) do
			alien.speed = 0.3 * (math.abs(20 - #aliens) + 1)
			alien.body:applyForce(alien.direction * alien.speed, 0)
			if alien.body:getX() < _WORLD_SIZE_X / 3 and alien.direction < 0 then
				alien.direction = alien.direction * -1
			elseif alien.body:getX() > _WORLD_SIZE_X - _WORLD_SIZE_X / 3 and alien.direction > 0 then
				alien.direction = alien.direction * -1
			end
			if alien.body:getY() < 0 then
				removeAlien(alien, i)
				modify_score(1)
			elseif alien.body:getY() > _WORLD_SIZE_Y - _GROUND_SIZE_Y - 10 then
				takeLife()
				removeAlien(alien, i)
--				modify_score(-1)
			end
		end
		if #aliens <= 0 then
			endGame("Victory\nlmb: New game")
		end
		mothership.speed = math.abs(20 - #aliens) + 1
		mothership.body:applyForce(mothership.direction * mothership.speed, 0)
		if mothership.body:getX() < _WORLD_SIZE_X / 3 and mothership.direction < 0 then
			mothership.direction = mothership.direction * -1
		elseif mothership.body:getX() > _WORLD_SIZE_X - _WORLD_SIZE_X / 3 and mothership.direction > 0 then
			mothership.direction = mothership.direction * -1
		end
		if mothership.latestshot < love.timer.getTime() - (1.5 - 1/#aliens) then
			mothershoot(mothership.body:getX(), mothership.body:getY())
			mothership.latestshot = love.timer.getTime()
		end
		-- key inputs
		if love.keyboard.isDown("left") then
			player.body:setX(player.body:getX() - 3)
		elseif love.keyboard.isDown("right") then
			player.body:setX(player.body:getX() + 3)
		end
		if love.keyboard.isDown(" ") then
			if player.latestShot < love.timer.getTime() - .25 then -- .25 -> max four shots per second
				if playerBullets > 0 then
					shoot()
					player.latestShot = love.timer.getTime()
				end
			end
		end
		for i, mes in ipairs(player.shootText) do
			if mes.born < love.timer.getTime() - .25 then
				table.remove(player.shootText, i)
			end
		end
		angle = player.body:getAngle()
		if(angle > 1) then
			player.body:setAngle(angle - 1)
		end
		if(angle < -1) then
			player.body:setAngle(angle + 1)
		end
		if scoremessage.size > 1 then
			scoremessage.size = scoremessage.size - .1
		end
	end--endif not pause
end
function takeLife()
	sound.rew_play(sounds.player.die)
	if (lives < 1) then endGame("The end\nlmb: New game") end
	lives = lives - 1
end
function shoot()
--	if #bullets>100 then
--		removeBullet(bullets[1], 1)
--	end
	playerBullets = playerBullets - 1
	sound.rew_play(sounds.player.shoot)
	bullet = {}
	bullet.radius = 5
	bullet.age = 0
	bullet.lastHitTime = 0
	bullet.blast = {}
	bullet.color = colors[love.math.random(1, #colors)]
	bullet.body = love.physics.newBody(world, player.body:getX(), player.body:getY() - 10, "dynamic")
	bullet.blast.x = bullet.body:getX()
	bullet.blast.y = bullet.body:getY() - 10
	bullet.blast.width = 15
	bullet.blast.height = 25
	bullet.blast.sizeMod = 1
	bullet.blast.age = 0
	bullet.blast.image = images.blast
	bullet.blast.color = _WHITE
	bullet.body:setBullet(true)
	bullet.image = images.bullet.happy
	bullet.shape = love.physics.newCircleShape(bullet.radius)
	bullet.fixture = love.physics.newFixture(bullet.body, bullet.shape, 1)
	bullet.body:applyForce(10 * (math.random() * 2 - 1), -900)
	bullet.body:applyTorque(love.math.random(-500, 500))
	table.insert(bullets, bullet)
	bullet.userData = {}
	bullet.userData.type = types.bullet
	bullet.userData.no = #bullets
	bullet.fixture:setUserData(bullet.userData)
	message = {}
	message.x = player.body:getX() + love.math.random(-40, 40)
	message.y = player.body:getY() + love.math.random(-40, -20)
	message.r = love.math.random(-25, 25)/100
	message.born = love.timer.getTime()
	table.insert(player.shootText, message)
end
-- remove one of the player's bullets
-- bullet: the bullet to remove TODO: fix crash
--function removeBullet(bullet, i)
--	bullet.fixture:destroy()
--	bullet.body:destroy()
--	table.remove(bullets, i)
--end
function makeAlien(x, y)
	alien = {}
	alien.color = colors[love.math.random(1, #colors)]
	alien.image = images.aliens[love.math.random(1, #images.aliens)]
	alien.body = love.physics.newBody(world, x, y, "dynamic")
	alien.width = 20
	alien.height = 10
	alien.shape = love.physics.newRectangleShape(20, 10)
	alien.fixture = love.physics.newFixture(alien.body, alien.shape, 1)
	alien.userData = {}
	alien.userData.type = types.alien
	alien.fixture:setUserData(alien.userData)
	alien.body:setGravityScale(0.005)
	alien.direction = -1
	alien.speed = .5
	table.insert(aliens, alien)
end
function removeAlien(alien, i)
	alien.fixture:destroy()
	alien.body:destroy()
	table.remove(aliens, i)
end
function endGame(message)
	endmessage = message
	pause = true
end
function sendMothership()
	mothership.body:setX(love.math.random(40, _WORLD_SIZE_X-40))
	mothership.body:setY(20)
end
function begincontact(a, b, coll)
	data_a = a:getUserData()
	if(data_a.type == types.bullet) then
		bullets[data_a.no].image = images.bullet.ow
		bullets[data_a.no].lastHitTime = love.timer.getTime()
	end
	data_b = b:getUserData()
	if(data_b.type == types.bullet) then
		bullets[data_b.no].image = images.bullet.ow
		bullets[data_b.no].lastHitTime = love.timer.getTime()
	end
	if(data_a.type == types.bullet and data_b.type == types.alien or data_a.type == types.alien and data_b.type == types.bullet) then
		sound.rew_play(sounds.bullet.hitAlien)
	end
end
function game.restartGame()
	lives = maxPlayerLives
	playerBullets = maxPlayerBullets
	score = 0
	scoremessage = {}
	scoremessage.size = 1
	scoremessage.inflatetime = .5
	scoremessage.color = {195, 195, 100}
	endmessage = "Pause\nesc: Continue\nlmb: New game\n\nControls:\n\tleft/right arrow: move\n\tspace: shoot\n\ts: toggle sound\n\nTo win shoot all the small aliens\nout of the top of the screen while\nkeeping them from reaching the ground."
	world = love.physics.newWorld(0, 9.81*64, true)
	world:setCallbacks(begincontact, endcontact, preSolve, postSolve)
	pause = false
	-- create player
	player = {}
	player.color = _WHITE
	player.width = 40
	player.height = 20
	player.image = images.player
	player.body = love.physics.newBody(world, _WORLD_SIZE_X/2, _WORLD_SIZE_Y - _GROUND_SIZE_Y - 100, "dynamic")
	player.shape = love.physics.newPolygonShape(-20, 0, 20, 0, 10, 10, -10, 10)
	player.fixture = love.physics.newFixture(player.body, player.shape, 3)
	player.userData = {}
	player.userData.type = types.player
	player.fixture:setUserData(player.userData)
	player.latestShot = 0
	player.shootText = {}
	bullets = {}
	alienBullets = {}
	-- create space aliens
	aliens = {}
	for i = 1, 20 do
		makeAlien(_WORLD_SIZE_X * love.math.random(), 15 * (i + 1))--100 * math.random())
	end
	mothership = {}
	mothership.latestshot = love.timer.getTime()
	mothership.direction = 1
	mothership.speed = 10
	mothership.color = colors[love.math.random(1, #colors)]
	mothership.image = images.alien_mother
	mothership.width = 40
	mothership.height = 20
	mothership.body = love.physics.newBody(world, 325, 20, "dynamic")
	mothership.shape = love.physics.newRectangleShape(40, 20)
	mothership.fixture = love.physics.newFixture(mothership.body, mothership.shape, 1)
	mothership.userData = {}
	mothership.userData.type = types.mothership
	mothership.fixture:setUserData(mothership.userData)
	mothership.body:setGravityScale(0)
	mothership.body:setAngularDamping(100)
	-- create static world objects
	worldobjects = {}
	worldobjects.ground = {}
	worldobjects.ground.chcoord = {}
	for x = 0, 65 do
		table.insert(worldobjects.ground.chcoord, 10*x)
		table.insert(worldobjects.ground.chcoord, love.math.random(-10, 0))
	end
	worldobjects.ground.body = love.physics.newBody(world, 0, _WORLD_SIZE_Y)
	worldobjects.ground.shape = love.physics.newChainShape(false, unpack(worldobjects.ground.chcoord))
	for x = 1, #worldobjects.ground.chcoord do
		if (x % 2 == 0) then
			worldobjects.ground.chcoord[x] = worldobjects.ground.chcoord[x] + _WORLD_SIZE_Y
		end
	end
	worldobjects.ground.fixture = love.physics.newFixture(worldobjects.ground.body, worldobjects.ground.shape)
	worldobjects.wallL = {}
	worldobjects.wallL.body = love.physics.newBody(world, 0, _WORLD_SIZE_Y/2)
	worldobjects.wallL.shape = love.physics.newRectangleShape(_WALL_SIZE_X, _WALL_SIZE_Y)
	worldobjects.wallL.fixture = love.physics.newFixture(worldobjects.wallL.body, worldobjects.wallL.shape)
	worldobjects.wallR = {}
	worldobjects.wallR.body = love.physics.newBody(world, _WORLD_SIZE_X, _WORLD_SIZE_Y/2)
	worldobjects.wallR.shape = love.physics.newRectangleShape(_WALL_SIZE_X, _WALL_SIZE_Y)
	worldobjects.wallR.fixture = love.physics.newFixture(worldobjects.wallR.body, worldobjects.wallR.shape)
	-- create static world objects
	worldUserData = {}
	worldUserData.type = types.world
	worldobjects.ground.fixture:setUserData(worldUserData)
	worldobjects.wallL.fixture:setUserData(worldUserData)
	worldobjects.wallR.fixture:setUserData(worldUserData)
	-- sky
	love.graphics.setBackgroundColor(_SKYCOLOR)
	love.window.setMode(_WORLD_SIZE_X, _WORLD_SIZE_Y, {fullscreen = false})
	sendMothership()
	pause = true
end
function mothershoot(x, y)
	bullet = {}
	bullet.radius = 5
	bullet.age = love.timer.getTime()
	bullet.body = love.physics.newBody(world, x, y + 20, "dynamic")
	bullet.body:applyForce(10 *(math.random() * 2 - 1), 0)
	bullet.body:applyTorque(love.math.random(-10, 10))
	bullet.color = colors[love.math.random(1, #colors)]
	bullet.image = images.alien_mother_bullet
	bullet.shape = love.physics.newCircleShape(bullet.radius)
	bullet.fixture = love.physics.newFixture(bullet.body, bullet.shape, 1)
	table.insert(alienBullets, bullet)
	bullet.userData = {}
	bullet.userData.type = types.alienBullet
	bullet.fixture:setUserData(bullet.userData)
	if #alienBullets > 10 then
		alienBullets[1].fixture:destroy()
		alienBullets[1].body:destroy()
		table.remove(alienBullets, 1)
	end
end
return game
